#include "machineReassignment.h"
#include <csignal>

static const string TeamIdentifier = "S21";

// ========== main ==========
int main(int argc, char *argv[]){
  string modelFileName, assignmentFileName, solutionFileName;
  int randomSeed(1);
  double maxCpuTime(300);

  // options & program parameters
  for (int i = 1; i < argc; i++){
    if (!strcmp(argv[i], "-t") && i+1 < argc){ maxCpuTime = max(0.0, atof(argv[++i])); }
    else if (!strcmp(argv[i], "-p") && i+1 < argc){ modelFileName = argv[++i]; }
    else if (!strcmp(argv[i], "-i") && i+1 < argc){ assignmentFileName = argv[++i]; }
    else if (!strcmp(argv[i], "-o") && i+1 < argc){ solutionFileName = argv[++i]; }
    else if (!strcmp(argv[i], "-name")){ 
      cout << TeamIdentifier << endl; 
      if (i == 1 && argc == 2) exit(1); 
    }
    else if (!strcmp(argv[i], "-s") && i+1 < argc){
      randomSeed = atoi(argv[++i]); 
    }
    else{
      cout << "Usage: " << argv[0]
        << " -t time_limit"
        << " -p instance_filename"
        << " -i original_solution_filename"
        << " -o new_solution_filename"
        << " -name"
        << " -s seed" 
        << endl;
      return 0;
    }
  }
  try{
    if (modelFileName.empty() || assignmentFileName.empty() || solutionFileName.empty()){
      throw runtime_error("Three filenames must be specified.");
    }
    MachineReassignment *mr = new MachineReassignment;
    mr->setFileNames(modelFileName, assignmentFileName, solutionFileName);
    mr->setMaxCpuTime(maxCpuTime);
    //mr->setRandomSeed(randomSeed);
    mr->setRandomSeed(20);
    mr->readProblem();
    mr->solve();
    mr->outputSolution();
  }
  catch (runtime_error error){
    cout << error.what() << endl;
    exit(1);
  }
}
