#include "machineReassignment.h"

MachineReassignment::int64 MachineReassignment::ProcessMachineMovingCost::calculatePenalty(Solution &solution){
  penalty = costs[solution.assignment[process]];
  return penalty;
}

MachineReassignment::int64 MachineReassignment::ProcessMachineMovingCost::updatePenalty(Solution &solution, Solver::Move *move){
  ppenalty = penalty;
  switch(move->type){
  case Solver::Shift:
    {
      Solver::ShiftMove *shift = static_cast<Solver::ShiftMove*>(move);
      if (shift->process != process) throw runtime_error("Error in ProcessMachineMovingCost::updatePenalty");
      penalty = costs[shift->machine];
      break;
    }
  case Solver::Swap:
    {
      Solver::SwapMove *swap = static_cast<Solver::SwapMove*>(move);
      if (swap->process1 == process) penalty = costs[solution.assignment[swap->process2]];
      else if (swap->process2 == process) penalty = costs[solution.assignment[swap->process1]];
      break;
    }
  }
  return penalty - ppenalty;
}

void MachineReassignment::ProcessMachineMovingCost::calculateDiff(Solution &solution){
  // calculate diff
  // shift
  unsigned int m0 = solution.assignment[process];
  for (unsigned int m = 0; m < con->mr->numMachines; ++m){
    if (m != m0) setShiftDiffPenalty(process, m, costs[m]-costs[m0]);
  }
  // swap
  vector<Solver::SwapMove*> &moves =  con->solver->moves.swapMovesByProcess[process];
  for (vector<Solver::SwapMove*>::iterator ite = moves.begin(); ite != moves.end(); ++ite){
    if (solution.assignment[(*ite)->process1] != solution.assignment[(*ite)->process2]){
      unsigned int process2;
      if ((*ite)->process1 == process) process2 = (*ite)->process2;
      else process2 = (*ite)->process1;
      setSwapDiffPenalty(*ite, costs[solution.assignment[(*ite)->process2]] - penalty);
    }
  }
}

void MachineReassignment::ProcessMachineMovingCost::prepare(MachineReassignment::Solver *solver){
  con->addConstraint(this);
  MachineReassignment *mr = solver->mr;
  process = mr->processes[process0].id2;
  costs.resize(mr->numMachines);
  for (unsigned int m = 0; m < mr->numMachines; ++m){
    unsigned int m0 = solver->processes[process].originalAssignment;
    costs[m] = mr->weightMachineMovingCost * mr->machines[m0].movingCosts[m];
    if (m != m0){
      costs[m] += mr->weightProcessMovingCost * solver->processes[process].movingCost;
    }
  }
  involve(process, inf);
}

// ----- Process/Machine Moving Costs -----
void MachineReassignment::Constraints::initProcessMachineMovingCosts(){
  processMachineMovingCostsTotal0 = 0;
  for (unsigned int p = 0; p < mr->numProcesses; ++p){
    ProcessMachineMovingCost *cost = new ProcessMachineMovingCost(this, 1, p);
    processMachineMovingCosts.push_back(cost);
    unsigned int m = mr->solution.assignment[p];
    unsigned int m0 = mr->processes[p].originalAssignment;
    if (m != m0){
      processMachineMovingCostsTotal0 += 
          mr->weightMachineMovingCost * mr->machines[m0].movingCosts[m]
          + mr->weightProcessMovingCost * mr->processes[p].movingCost;
    }
  }
}
void MachineReassignment::Constraints::prepareProcessMachineMovingCosts(){
  processMachineMovingCostsTotalBackup = 0;
  for (unsigned int p = 0; p < solver->numProcesses; ++p){
    ProcessMachineMovingCost *cost = processMachineMovingCosts[solver->processes[p].id];
    cost->prepare(solver);
    processMachineMovingCostsTotalBackup += cost->calculatePenalty(solver->current);
  }
}
void MachineReassignment::Constraints::postprocessProcessMachineMovingCosts(Solution &solution){
  processMachineMovingCostsTotal0 -= processMachineMovingCostsTotalBackup;
  for (unsigned int p = 0; p < solver->numProcesses; ++p){
    processMachineMovingCostsTotal0 += processMachineMovingCosts[solver->processes[p].id]->calculatePenalty(solution);
  }
}